--[[
	Data Parsed from WowHead.com and tradeskillmaster.com
	Locales: US, ES, MX, DE, FR, RU
	]]--

SmexyMats = LibStub("AceAddon-3.0"):NewAddon("SmexyMats", "AceEvent-3.0", "AceConsole-3.0");

local L = LibStub("AceLocale-3.0"):GetLocale("SmexyMats");
local AceConfig = LibStub("AceConfigDialog-3.0");
local name = "SmexyMats(SM)"; 
local version = "v8.0.1.82"; 
local isTooltipDone = nil;
local storedLink = nil;
local LID = nil;

function SmexyMats:OnInitialize()
	SmexyMats:RegisterChatCommand("sm", "ChatCommand");
	SmexyMats.db = LibStub("AceDB-3.0"):New("SmexyMatsDB", SmexyMats.defaults, "profile");
	LibStub("AceConfig-3.0"):RegisterOptionsTable("SmexyMats", SmexyMats.options);
	AceConfig:AddToBlizOptions("SmexyMats", "SmexyMats(SM)");
	local tooltipMethodHooks = {
		SetCurrencyToken = { nil, Hook_SetCurrencyToken },
		SetQuestItem = { function(self,type,index) storedLink = GetQuestItemLink(type,index) end, nil },
		SetQuestLogItem = { function(self,type,index) storedLink = GetQuestLogItemLink(type,index) end, nil },
		SetRecipeResultItem = { function(self, recipeID) if recipeID then storedLink = C_TradeSkillUI.GetRecipeItemLink(recipeID) end end, nil},
		SetRecipeReagentItem = { function(self, recipeID, reagentIndex) if recipeID and reagentIndex then storedLink = C_TradeSkillUI.GetRecipeReagentItemLink(recipeID, reagentIndex) end end, nil },
	};
	for m, hooks in pairs(tooltipMethodHooks) do
		InstallHook(GameTooltip, m, hooks[1], hooks[2]);
	end;
	if (SmexyMats:CheckDB() == true) then
		SmexyMats:HookTooltips(); 
		print(SmexyMats.Colors.wowtoken .. name,SmexyMats.Colors.legendary .. version,L["|rLoad Complete!"]);
	else
		print(SmexyMats.Colors.wowtoken .. name,SmexyMats.Colors.legendary .. version,SmexyMats.Colors.DeathKnight .. L["Failed! |rMissing Data-Tables. Reinstall SemxyMats(SM) to correct this issue or report the error to https://mods.curse.com/addons/wow/270824-smexymats"]);
	end;
end;	

function GetItemProperties(item)
	local retObj = {}; 
	if not item then return; end;
	
	retObj.aa, retObj.bb, retObj.cc, retObj.dd, retObj.ee, retObj.ff, retObj.gg, retObj.hh, retObj.ii, retObj.jj, retObj.kk, retObj.ll, retObj.mm, retObj.nn, retObj.oo, retObj.pp = GetItemInfo(item);
	
	if retObj.bb then retObj.ID = GetIDFromLink(retObj.bb); end;
	
	return retObj;
end;

function SmexyMats:HookTooltips()
	--Install all of the hooks...
	GameTooltip:HookScript("OnShow", JustTheTip);
	GameTooltip:HookScript("OnTooltipCleared", function(self) isTooltipDone = nil; end);
	GameTooltip:HookScript("OnTooltipSetItem", SmexyMats.ModifyItemTooltip);
	ItemRefTooltip:HookScript("OnShow", JustTheTip);
	ItemRefTooltip:HookScript("OnTooltipCleared", function(self) isTooltipDone = nil; end);
	ItemRefTooltip:HookScript("OnTooltipSetItem", SmexyMats.ModifyItemTooltip);
end;

function SmexyMats:ChatCommand()
	InterfaceOptionsFrame_OpenToCategory("SmexyMats(SM)");
	InterfaceOptionsFrame_OpenToCategory("SmexyMats(SM)");
end;

function JustTheTip(tooltip, ...)
	tooltip:SetBackdropColor(0,0,0,1); 
	tooltip:Show();
end;

function GetIDFromLink(link)
	if link then
		local linktype, id = string.match(link, "|H([^:]+):(%d+)");
		if id then 
			return tonumber(id); 
		else 
			return 0; 
		end;
	end;
end;

function InstallHook(tooltip, method, prehook, posthook)
	local orig = tooltip[method];
	local stub = function(...)
		if prehook then 
			prehook(...); 
		end;
		local a,b,c,d,e,f,g,h,i,j,k = orig(...);
		if posthook then 
			posthook(...); 
		end;
		return a,b,c,d,e,f,g,h,i,j,k;
	end;
	tooltip[method] = stub;
end;

function SmexyMats:CheckDB()
	if not (SmexyMats.Sources) then
		return false;
	end;
	if not (SmexyMats.Reagents) then
		return false;
	end;
	return true;
end;

function SmexyMats:GetExPack(obj)
	if (obj.aa == nil) or (obj.aa == '') or (string.len(obj.aa) < 1) then 
		return -1, SmexyMats.ExPacks[-1].color; 
	end;

	if (SmexyMats.db.profile.Equipment == true) and (SmexyMats.db.profile.Contents == true) then 
		local X, Y = SmexyMats:Gear_ExpackID(obj); 
		if (X ~= -1) then 
			return X, Y; 
		end; 
	end;
	return -1, SmexyMats.ExPacks[-1].color;
end;

function ProcessTooltip(tt, obj)
	local ItemInfoCached, r, g, b = true, .9, .8, .5;
	if not obj.ID then 
		ItemInfoCached = false; 
	end;
	if (obj.ID == 0) and (TradeSkillFrame ~= nil) and TradeSkillFrame:IsVisible() then
		if (GetMouseFocus():GetName()) == "TradeSkillSkillIcon" then
			obj.ID = tonumber(GetTradeSkillItemLink(TradeSkillFrame.selectedSkill):match("item:(%d+):")) or nil
		else
			for i = 1, 8 do
				if (GetMouseFocus():GetName()) == "TradeSkillReagent"..i then
					obj.ID = tonumber(GetTradeSkillReagentItemLink(TradeSkillFrame.selectedSkill, i):match("item:(%d+):")) or nil
					break;
				end;
			end;
		end;
	end;
	if (obj.ID == 0) then 
		ItemInfoCached = false; 
	end;
	if (not ItemInfoCached) then
		if (SmexyMats.db.profile.ErrorReporting == true) then
			if (_G["GameTooltipTextLeft1"]:GetText() ~= SmexyMats.LID) then
				print(SmexyMats.Colors.wowtoken .. L["[SM]:"] .. SmexyMats.Colors.DeathKnight .. L["Cannot fetch information for:"]);
				print(SmexyMats.Colors.wowtoken .. L["[SM]:"] .. SmexyMats.Colors.common ..  _G["GameTooltipTextLeft1"]:GetText());
				print(SmexyMats.Colors.wowtoken .. L["[SM]:"] .. SmexyMats.Colors.yellow .. L["An info request has been sent to the server for details on this item."]);
				print(SmexyMats.Colors.wowtoken .. L["[SM]:"] .. SmexyMats.Colors.uncommon .. L["Temporary Solution:"]);
				print(SmexyMats.Colors.wowtoken .. L["[SM]:"] .. SmexyMats.Colors.yellow .. L["1. Link the item in chat."]);
				print(SmexyMats.Colors.wowtoken .. L["[SM]:"] .. SmexyMats.Colors.yellow .. L["2. Click on it."]);
				print(SmexyMats.Colors.wowtoken .. L["[SM]:"] .. SmexyMats.Colors.uncommon .. L["To turn off these errors:"]);
				print(SmexyMats.Colors.wowtoken .. L["[SM]:"] .. SmexyMats.Colors.yellow .. L["type /sm and uncheck 'Report errors to chat'."]);
				print(SmexyMats.Colors.wowtoken .. L["[SM]:"] .. SmexyMats.Colors.uncommon .. L["More Info:"]);
				print(SmexyMats.Colors.wowtoken .. L["[SM]:"] .. SmexyMats.Colors.yellow .. L["More info on why this error occured can be found here:"]);
				print(SmexyMats.Colors.Mage .. "http://wowprogramming.com/docs/api/GetItemInfo");
				LID = _G["GameTooltipTextLeft1"]:GetText();
			end;
		end;
		return;
	end;
	
	local ProFor, ProFrom, EP;
	
	if (SmexyMats:Gear_ExpackID(obj)) == true then
		qwe, EP = SmexyMats:Gear_ExpackID(obj)
	else
		ProFor, ProFrom, EP = SmexyMats:FormatToolTipString(obj.ID);
	end;
	
	local tttE, tttS, tttP, tttI;
	if (SmexyMats.db.profile.SMText == true) then
		tttE = L["[SM]Expansion: "];
		tttS = L["[SM]Source(s): "];
		tttP = L["[SM]Profession(s): "];
		tttI = L["[SM]ItemID: "];
	else	
		tttE = L["Expansion: "];
		tttS = L["Source(s): "];
		tttP = L["Profession(s): "];
		tttI = L["ItemID: "];
	end;
	
	tt:AddLine(" ",1,1,1);
	if (SmexyMats.db.profile.Contents == true) then 
		if EP ~= nil then
			local e = SmexyMats.ExPacks[EP].name; 
			
			tt:AddDoubleLine(SmexyMats.Colors.wowtoken .. tttE,SmexyMats.ExPacks[EP].color .. e,r,b,g,1,1,1,true);
		end; 
	end;
	if (SmexyMats.db.profile.Sources == true) then
		if (obj.aa) and (ProFrom) and (#(ProFrom) > 0) then 
			tt:AddLine(SmexyMats.Colors.wowtoken .. tttS .. SmexyMats.Colors.white .. ProFrom,1,1,1,true);
		end;
	end;
	if (SmexyMats.db.profile.Professions == true) then
		if (obj.aa) and (ProFor) and (#(ProFor) > 0) then 
			tt:AddLine(SmexyMats.Colors.wowtoken .. tttP .. SmexyMats.Colors.white .. ProFor,1,1,1,true);
		end;
	end;
	if (SmexyMats.db.profile.ItemIDs == true) then 
		tt:AddDoubleLine(SmexyMats.Colors.wowtoken .. tttI,SmexyMats.Colors.Paladin .. obj.ID,r,b,g,1,1,1,true);
	end;
end;

function ExamineObject(obj)
	if not obj then 
		return false; 
	end; 
	if (not obj.aa) or (obj.aa == "") or (not obj.bb) or (not obj.ID) or (obj.ID == 0) then 
		return false; 
	end; 
	return true;
end;

function SmexyMats.ModifyItemTooltip(tt, ...)
	if (SmexyMats.db.profile.Enabled == false) then 
		return; 
	end;
	local obj, name, link, itemAquired = nil, nil, nil, false;
	local objTXT = _G["GameTooltipTextLeft1"]:GetText();
	if (not isTooltipDone) and tt then
		isTooltipDone = true;
		name, link = tt:GetItem();
		obj = GetItemProperties(link)
		if (ExamineObject(obj)) and (ExamineObject(obj)) then
			itemAquired = true;
		else
			link = storedLink; 
			obj = GetItemProperties(link);
			if (ExamineObject(obj) == true) and (ExamineObject(obj) == true) then 
				itemAquired = true;
			else
				obj = GetItemProperties(objTXT);
				if (ExamineObject(obj)) and (ExamineObject(obj)) then 
					itemAquired = true; 
				end;
			end;
		end;
		if itemAquired then ProcessTooltip(tt, obj); end;
	end;
end;

function SmexyMats:FormatToolTipString(iID)
	local zForTTL, zFromTTL, zEP = SmexyMats:SearchDatabase(iID); 
	local zForTTS = "";
	local zFromTTS = "";
	
	table.sort(zForTTL);
	for k, v in pairs(zForTTL) do 
		if (zForTTS == "") then 
			zForTTS = SmexyMats:trim(v); 
		else
			if not (string.match(zForTTS, SmexyMats:trim(v))) then
				zForTTS = zForTTS .. ", " .. SmexyMats:trim(v); 
			end;
		end;
	end;
	
	table.sort(zFromTTL);
	for k, v in pairs(zFromTTL) do 
		if (zFromTTS == "") then 
			zFromTTS = SmexyMats:trim(v); 
		else 
			if not(string.match(zFromTTS, SmexyMats:trim(v))) then
				zFromTTS = zFromTTS .. ", " .. SmexyMats:trim(v); 
			end;
		end;
	end;
	
	return zForTTS, zFromTTS, zEP;
end;

function SmexyMats:SearchDatabase(iID)
	local xForTTL = {};
	local xFromTTL = {};
	local zz = nil;
	local xpackfound, ypackfound = false, false;
	
	if not iID then return; end;
	
	--Loops through Professions
	for x = 1, 14 do
		--Loops through Expansions
		for y = 0, 7 do
			--Loops through Sources
			for k, v in pairs( SmexyMats.Sources[x][y] ) do 
				if (tonumber(iID) == v) then
					table.insert(xFromTTL, SmexyMats.Profs[x]);
					if (zz == nil) then
						zz = y;
					end;
				end;
			end;
			--Loops through Reagents
			for l, w in pairs( SmexyMats.Reagents[x][y] ) do 
				if (tonumber(iID) == w) then
					table.insert(xForTTL, SmexyMats.Profs[x]);
					if (zz == nil) then
						zz = y;
					end;
				end;
			end;
		end;
	end;
	--Loops through Vendor Mats
	for _c, _d in pairs( SmexyMats.Vendor ) do
		if (tonumber(iID)) == _d then
			table.insert(xFromTTL, SmexyMats.Profs[-1]);
		end;
	end;
	--Loops through Drops
	for _e, _f in pairs( SmexyMats.Drop ) do
		if (tonumber(iID)) == _f then
			table.insert(xFromTTL, SmexyMats.Profs[-2]);
		end;
	end;
	--Loops through Scraps
	for _g, _h in pairs( SmexyMats.Scrap ) do
		if (tonumber(iID)) == _h then
			table.insert(xFromTTL, SmexyMats.Profs[-3]);
		end;
	end;
	--Returns Results
	return xForTTL, xFromTTL, zz;
end;